@@ -62,6 +62,7 @@ var LibraryManager = {
     // Core system libraries (always linked against)
     var libraries = [
       'library.js',
+      'library_stack.js',
       'library_formatString.js',
       'library_math.js',
       'library_path.js',
@@ -93,6 +94,7 @@ var LibraryManager = {
 
     if (USE_PTHREADS) { // TODO: Currently WebGL proxying makes pthreads library depend on WebGL.
       libraries.push('library_webgl.js');
+      libraries.push('library_html5_webgl.js');
     }
 
     if (FILESYSTEM) {
@@ -118,6 +120,7 @@ var LibraryManager = {
     if (AUTO_JS_LIBRARIES) {
       libraries = libraries.concat([
         'library_webgl.js',
+        'library_html5_webgl.js',
         'library_openal.js',
         'library_sdl.js',
         'library_glut.js',
@@ -130,14 +133,14 @@ var LibraryManager = {
         'library_async.js'
       ]);
     } else {
-      if (EMTERPRETIFY_ASYNC || ASYNCIFY) {
+      if (ASYNCIFY) {
         libraries.push('library_async.js');
       }
       if (USE_SDL == 1) {
         libraries.push('library_sdl.js');
       }
       if (USE_SDL == 2) {
-        libraries.push('library_egl.js', 'library_webgl.js');
+        libraries.push('library_egl.js', 'library_webgl.js', 'library_html5_webgl.js');
       }
     }
 
@@ -158,10 +161,16 @@ var LibraryManager = {
 
     if (USE_WEBGPU) {
       libraries.push('library_webgpu.js');
+      libraries.push('library_html5_webgpu.js');
     }
 
     if (BOOTSTRAPPING_STRUCT_INFO) {
-      libraries = ['library_formatString.js'];
+      libraries = [
+        'library_bootstrap.js',
+        'library_stack.js',
+        'library_formatString.js',
+        'library_stack_trace.js'
+      ];
     }
 
     // Deduplicate libraries to avoid processing any library file multiple times
@@ -480,9 +489,6 @@ function exportRuntime() {
   if (STACK_OVERFLOW_CHECK) {
     runtimeElements.push('writeStackCookie');
     runtimeElements.push('checkStackCookie');
-    if (!MINIMAL_RUNTIME) {
-      runtimeElements.push('abortStackOverflow');
-    }
   }
 
   if (USE_PTHREADS) {
