@@ -122,7 +122,9 @@ var TOTAL_STACK = 5*1024*1024;
 // In general, if you don't need one of those special modes, and if you don't
 // allocate very many small objects, you should use emmalloc since it's
 // smaller. Otherwise, if you do allocate many small objects, dlmalloc
-// is usually worth the extra size.
+// is usually worth the extra size. dlmalloc is also a good choice if you want
+// the extra security checks it does (such as noticing metadata corruption in
+// its internal data structures, which emmalloc does not do).
 var MALLOC = "dlmalloc";
 
 // If 1, then when malloc would fail we abort(). This is nonstandard behavior,
@@ -286,14 +288,6 @@ var WARN_UNALIGNED = 0;
 // behavior with WebAssembly.
 var PRECISE_F32 = 0;
 
-// Whether to allow autovectorized SIMD code
-// (https://github.com/johnmccutchan/ecmascript_simd).  SIMD intrinsics are
-// always compiled to SIMD code, so you only need this option if you also want
-// the autovectorizer to run.  Note that SIMD support in browsers is not yet
-// there (as of Sep 2, 2014), so you will be running in a polyfill, which is not
-// fast.
-var SIMD = 0;
-
 // Whether closure compiling is being run on this output
 var USE_CLOSURE_COMPILER = 0;
 
@@ -335,7 +329,7 @@ var SIMPLIFY_IFS = 1;
 
 // Check each write to the heap, for example, this will give a clear
 // error on what would be segfaults in a native build (like dereferencing
-// 0). See preamble.js for the actual checks performed.
+// 0). See runtime_safe_heap.js for the actual checks performed.
 var SAFE_HEAP = 0;
 
 // Log out all SAFE_HEAP operations
@@ -663,7 +657,7 @@ var LZ4 = 0;
 // DISABLE_EXCEPTION_CATCHING = 0 - generate code to actually catch exceptions
 // DISABLE_EXCEPTION_CATCHING = 1 - disable exception catching at all
 // DISABLE_EXCEPTION_CATCHING = 2 - disable exception catching, but enables
-//                                  catching in whitelist
+//                                  catching in list of allowed functions
 // XXX note that this removes *catching* of exceptions, which is the main
 //     issue for speed, but you should build source files with
 //     -fno-exceptions to really get rid of all exceptions code overhead,
@@ -676,7 +670,7 @@ var DISABLE_EXCEPTION_CATCHING = 1;
 // Enables catching exception in the listed functions only, if
 // DISABLE_EXCEPTION_CATCHING = 2 is set
 // [compile+link] - affects user code at compile and system libraries at link
-var EXCEPTION_CATCHING_WHITELIST = [];
+var EXCEPTION_CATCHING_ALLOWED = [];
 
 // By default we handle exit() in node, by catching the Exit exception. However,
 // this means we catch all process exceptions. If you disable this, then we no
@@ -699,10 +693,10 @@ var NODEJS_CATCH_REJECTION = 1;
 // This allows to inject some async functions to the C code that appear to be sync
 // e.g. emscripten_sleep
 // On fastcomp this uses the Asyncify IR transform.
-// On upstream this uses the Asyncify pass in Binaryen. TODO: whitelist, coroutines
+// On upstream this uses the Asyncify pass in Binaryen.
 var ASYNCIFY = 0;
 
-// Imports which can do a sync operation, in addition to the default ones that
+// Imports which can do an sync operation, in addition to the default ones that
 // emscripten defines like emscripten_sleep. If you add more you will need to
 // mention them to here, or else they will not work (in ASSERTIONS builds an
 // error will be shown).
@@ -722,7 +716,7 @@ var ASYNCIFY_IGNORE_INDIRECT = 0;
 // In that case, you should increase this size.
 var ASYNCIFY_STACK_SIZE = 4096;
 
-// If the Asyncify blacklist is provided, then the functions in it will not
+// If the Asyncify remove-list is provided, then the functions in it will not
 // be instrumented even if it looks like they need to. This can be useful
 // if you know things the whole-program analysis doesn't, like if you
 // know certain indirect calls are safe and won't unwind. But if you
@@ -743,13 +737,22 @@ var ASYNCIFY_STACK_SIZE = 4096;
 // builds, etc.). You can inspect the wasm binary to look for the actual names,
 // either directly or using wasm-objdump or wasm-dis, etc.
 // Simple '*' wildcard matching is supported.
-var ASYNCIFY_BLACKLIST = [];
+var ASYNCIFY_REMOVE = [];
 
-// If the Asyncify whitelist is provided, then *only* the functions in the list
-// will be instrumented. Like the blacklist, getting this wrong will break
+// Functions in the Asyncify add-list are added to the list of instrumented
+// functions, that is, they will be instrumented even if otherwise asyncify
+// thinks they don't need to be. As by default everything will be instrumented
+// in the safest way possible, this is only useful if you use IGNORE_INDIRECT
+// and use this list to fix up some indirect calls that *do* need to be
+// instrumented.
+// See notes on ASYNCIFY_REMOVE about the names.
+var ASYNCIFY_ADD = [];
+
+// If the Asyncify only-list is provided, then *only* the functions in the list
+// will be instrumented. Like the remove-list, getting this wrong will break
 // your application.
-// See notes on ASYNCIFY_BLACKLIST about the names.
-var ASYNCIFY_WHITELIST = [];
+// See notes on ASYNCIFY_REMOVE about the names.
+var ASYNCIFY_ONLY = [];
 
 // Allows lazy code loading: where emscripten_lazy_load_code() is written, we
 // will pause execution, load the rest of the code, and then resume.
@@ -959,6 +962,12 @@ var PROXY_TO_WORKER_FILENAME = '';
 // something that applications can do manually as well if they wish, this option
 // is provided as convenience.
 //
+// The pthread that main() runs on is a normal pthread in all ways, with the one
+// difference that its stack size is the same as the main thread would normally
+// have, that is, TOTAL_STACK. This makes it easy to flip between
+// PROXY_TO_PTHREAD and non-PROXY_TO_PTHREAD modes with main() always getting
+// the same amount of stack.
+//
 // This proxies Module['canvas'], if present, and if OFFSCREEN_CANVAS support
 // is enabled. This has to happen because this is the only chance - this browser
 // main thread does the only pthread_create call that happens on
@@ -1037,7 +1046,11 @@ var SMALL_XHR_CHUNKS = 0;
 var HEADLESS = 0;
 
 // If 1, we force Date.now(), Math.random, etc. to return deterministic results.
-// Good for comparing builds for debugging purposes (and nothing else)
+// This also tries to make execution deterministic across machines and
+// environments, for example, not doing anything different based on the
+// browser's language setting (which would mean you can get different results
+// in different browsers, or in the browser and in node).
+// Good for comparing builds for debugging purposes (and nothing else).
 var DETERMINISTIC = 0;
 
 // By default we emit all code in a straightforward way into the output
@@ -1172,48 +1185,6 @@ var EXPORT_NAME = 'Module';
 // to warnings instead of throwing an exception.
 var DYNAMIC_EXECUTION = 1;
 
-// Runs tools/emterpretify on the compiler output.
-// [fastcomp-only]
-var EMTERPRETIFY = 0;
-
-// If defined, a file to write bytecode to, otherwise the default is to embed it
-// in text JS arrays (which is less efficient).  When emitting HTML, we
-// automatically generate code to load this file and set it to
-// Module.emterpreterFile. If you emit JS, you need to make sure that
-// Module.emterpreterFile contains an ArrayBuffer with the bytecode, when the
-// code loads.  Note: You might need to quote twice in the shell, something like
-// -s 'EMTERPRETIFY_FILE="waka"'
-// [fastcomp-only]
-var EMTERPRETIFY_FILE = '';
-
-// Functions to not emterpret, that is, to run normally at full speed
-// [fastcomp-only]
-var EMTERPRETIFY_BLACKLIST = [];
-
-// If this contains any functions, then only the functions in this list are
-// emterpreted (as if all the rest are blacklisted; this overrides the
-// BLACKLIST)
-// [fastcomp-only]
-var EMTERPRETIFY_WHITELIST = [];
-
-// Allows sync code in the emterpreter, by saving the call stack, doing an async
-// delay, and resuming it
-// [fastcomp-only]
-var EMTERPRETIFY_ASYNC = 0;
-
-// Performs a static analysis to suggest which functions should be run in the
-// emterpreter, as it appears they can be on the stack when a sync function is
-// called in the EMTERPRETIFY_ASYNC option.  After showing the suggested list,
-// compilation will halt. You can apply the provided list as an emcc argument
-// when compiling later.
-// [fastcomp-only]
-var EMTERPRETIFY_ADVISE = 0;
-
-// If you have additional custom synchronous functions, add them to this list
-// and the advise mode will include them in its analysis.
-// [fastcomp-only]
-var EMTERPRETIFY_SYNCLIST = [];
-
 // whether js opts will be run, after the main compiler
 var RUNNING_JS_OPTS = 0;
 
@@ -1242,11 +1213,14 @@ var USE_GLFW = 2;
 // In that build mode, two files a.wasm and a.wasm.js are produced, and at runtime
 // the WebAssembly file is loaded if browser/shell supports it. Otherwise the
 // .wasm.js fallback will be used.
+//
+// If WASM=2 is enabled and the browser fails to compile the WebAssembly module,
+// the page will be reloaded in Wasm2JS mode.
 var WASM = 1;
 
-// STANDALONE_WASM indicates that we want to emit a wasm file that can run without
-// JavaScript. The file will use standard APIs such as wasi as much as possible
-// to achieve that.
+// STANDALONE_WASM indicates that we want to emit a wasm file that can run
+// without JavaScript. The file will use standard APIs such as wasi as much as
+// possible to achieve that.
 //
 // This option does not guarantee that the wasm can be used by itself - if you
 // use APIs with no non-JS alternative, we will still use those (e.g., OpenGL
@@ -1267,6 +1241,18 @@ var WASM = 1;
 // environments the expectation is to create the memory in the wasm itself.
 // Doing so prevents some possible JS optimizations, so we only do it behind
 // this flag.
+//
+// When this flag is set we do not legalize the JS interface, since the wasm is
+// meant to run in a wasm VM, which can handle i64s directly. If we legalized it
+// the wasm VM would not recognize the API. However, this means that the
+// optional JS emitted won't run if you use a JS API with an i64. You can use
+// the WASM_BIGINT option to avoid that problem by using BigInts for i64s which
+// means we don't need to legalize for JS (but this requires a new enough JS
+// VM).
+//
+// Standlone builds by default require a `main` entry point.  If you want to
+// build a library (also known as a reactor) instead you can pass `--no-entry`
+// or specify a list of EXPORTED_FUNCTIONS that does not include `main`.
 var STANDALONE_WASM = 0;
 
 // Whether to use the WebAssembly backend that is in development in LLVM.  You
@@ -1341,7 +1327,6 @@ var EMIT_PRODUCERS_SECTION = 0;
 var EMIT_EMSCRIPTEN_METADATA = 0;
 
 // Emits emscripten license info in the JS output.
-// [upstream-only]
 var EMIT_EMSCRIPTEN_LICENSE = 0;
 
 // Whether to legalize the JS FFI interfaces (imports/exports) by wrapping them
@@ -1400,6 +1385,9 @@ var USE_VORBIS = 0;
 // 1 = use ogg from emscripten-ports
 var USE_OGG = 0;
 
+// 1 = use mpg123 from emscripten-ports
+var USE_MPG123 = 0;
+
 // 1 = use freetype from emscripten-ports
 var USE_FREETYPE = 0;
 
@@ -1416,6 +1404,9 @@ var USE_COCOS2D = 0;
 // Formats to support in SDL2_image. Valid values: bmp, gif, lbm, pcx, png, pnm, tga, xcf, xpm, xv
 var SDL2_IMAGE_FORMATS = [];
 
+// Formats to support in SDL2_mixer. Valid values: ogg, mp3
+var SDL2_MIXER_FORMATS = ["ogg"];
+
 // The list of defines (C_DEFINES) was moved into struct_info.json in the same
 // directory.  That file is automatically parsed by tools/gen_struct_info.py.
 // If you modify the headers, just clear your cache and emscripten libc should
@@ -1614,8 +1605,8 @@ var ASMFS = 0;
 //
 // When using code that depends on this option, your Content Security Policy may
 // need to be updated. Specifically, embedding asm.js requires the script-src
-// directive to whitelist 'unsafe-inline', and using a Worker requires the
-// child-src directive to whitelist blob:. If you aren't using Content Security
+// directive to allow 'unsafe-inline', and using a Worker requires the
+// child-src directive to allow blob:. If you aren't using Content Security
 // Policy, or your CSP header doesn't include either script-src or child-src,
 // then you can safely ignore this warning.
 var SINGLE_FILE = 0;
@@ -1795,6 +1786,19 @@ var DEFAULT_TO_CXX = 1;
 // long double printing precision.
 var PRINTF_LONG_DOUBLE = 0;
 
+// Run wabt's wasm2c tool on the final wasm, and combine that with a C runtime,
+// resulting in a .c file that you can compile with a C compiler to get a
+// native executable that works the same as the normal js+wasm. This will also
+// emit the wasm2c .h file. The output filenames will be X.wasm.c, X.wasm.h
+// if your output is X.js or X.wasm (note the added .wasm. we make sure to emit,
+// which avoids trampling a C file).
+var WASM2C = 0;
+
+// Setting this affects the path emitted in the wasm that refers to the DWARF
+// file, in -gseparate-dwarf mode. This allows the debugging file to be hosted
+// in a custom location.
+var SEPARATE_DWARF_URL = '';
+
 //===========================================
 // Internal, used for testing only, from here
 //===========================================
@@ -1849,4 +1853,7 @@ var LEGACY_SETTINGS = [
   ['BINARYEN_MEM_MAX', 'MAXIMUM_MEMORY'],
   ['BINARYEN_PASSES', [''], 'Use BINARYEN_EXTRA_PASSES to add additional passes'],
   ['SWAPPABLE_ASM_MODULE', [0], 'Fully swappable asm modules are no longer supported'],
+  ['ASYNCIFY_WHITELIST', 'ASYNCIFY_ONLY'],
+  ['ASYNCIFY_BLACKLIST', 'ASYNCIFY_REMOVE'],
+  ['EXCEPTION_CATCHING_WHITELIST', 'EXCEPTION_CATCHING_ALLOWED'],
 ];
